package WebDAO::Component;

=head1 NAME

WebDAO::Component - Component class

=head1 SYNOPSIS

=head1 DESCRIPTION

WebDAO::Component - Component class (deprecated)

=head1 METHODS

=cut

use WebDAO::Base;
use base qw(WebDAO::Element);
use strict 'vars';
use Data::Dumper;


1;
__DATA__

=head1 SEE ALSO

http://webdao.sourceforge.net

=head1 AUTHOR

Zahatski Aliaksandr, E<lt>zag@cpan.orgE<gt>

=head1 COPYRIGHT AND LICENSE

Copyright 2002-2014 by Zahatski Aliaksandr

This library is free software; you can redistribute it and/or modify
it under the same terms as Perl itself. 

=cut

